@@ -96,8 +96,7 @@ static void at_call_barring_query(struct ofono_call_barring *cb,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(callback, 0, data);
 }
@@ -139,8 +138,7 @@ static void at_call_barring_set(struct ofono_call_barring *cb, const char *lock,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(callback, data);
 }
@@ -177,8 +175,7 @@ static void at_call_barring_set_passwd(struct ofono_call_barring *cb,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(callback, data);
 }
@@ -197,7 +194,7 @@ static int at_call_barring_probe(struct ofono_call_barring *cb,
 {
 	GAtChat *chat = user;
 
-	ofono_call_barring_set_data(cb, chat);
+	ofono_call_barring_set_data(cb, g_at_chat_clone(chat));
 	g_idle_add(at_call_barring_register, cb);
 
 	return 0;
@@ -205,6 +202,10 @@ static int at_call_barring_probe(struct ofono_call_barring *cb,
 
 static void at_call_barring_remove(struct ofono_call_barring *cb)
 {
+	GAtChat *chat = ofono_call_barring_get_data(cb);
+
+	g_at_chat_unref(chat);
+	ofono_call_barring_set_data(cb, NULL);
 }
 
 static struct ofono_call_barring_driver driver = {
