@@ -27,6 +27,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <errno.h>
 
 #include <glib.h>
 
@@ -52,7 +53,6 @@ static void calypso_generic_cb(gboolean ok, GAtResult *result,
 	ofono_voicecall_cb_t cb = cbd->cb;
 	struct ofono_error error;
 
-	dump_response("calypso_generic_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	cb(&error, cbd->data);
@@ -72,8 +72,7 @@ static void calypso_template(struct ofono_voicecall *vc, const char *cmd,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, data);
 }
@@ -87,9 +86,9 @@ static void calypso_dial(struct ofono_voicecall *vc,
 	char buf[256];
 
 	if (ph->type == 145)
-		sprintf(buf, "ATD+%s", ph->number);
+		snprintf(buf, sizeof(buf), "ATD+%s", ph->number);
 	else
-		sprintf(buf, "ATD%s", ph->number);
+		snprintf(buf, sizeof(buf), "ATD%s", ph->number);
 
 	switch (clir) {
 	case OFONO_CLIR_OPTION_INVOCATION:
@@ -121,12 +120,18 @@ static void calypso_answer(struct ofono_voicecall *vc,
 	calypso_template(vc, "ATA", cb, data);
 }
 
-static void calypso_hangup(struct ofono_voicecall *vc,
+static void calypso_ath(struct ofono_voicecall *vc,
 				ofono_voicecall_cb_t cb, void *data)
 {
 	calypso_template(vc, "ATH", cb, data);
 }
 
+static void calypso_chup(struct ofono_voicecall *vc,
+				ofono_voicecall_cb_t cb, void *data)
+{
+	calypso_template(vc, "AT+CHUP", cb, data);
+}
+
 static void calypso_hold_all_active(struct ofono_voicecall *vc,
 					ofono_voicecall_cb_t cb, void *data)
 {
@@ -159,7 +164,7 @@ static void calypso_release_specific(struct ofono_voicecall *vc, int id,
 	/* On calypso, 1X only releases active calls, while 7X releases
 	 * active or held calls
 	 */
-	sprintf(buf, "AT%%CHLD=7%d", id);
+	snprintf(buf, sizeof(buf), "AT%%CHLD=7%d", id);
 	calypso_template(vc, buf, cb, data);
 }
 
@@ -168,7 +173,7 @@ static void calypso_private_chat(struct ofono_voicecall *vc, int id,
 {
 	char buf[32];
 
-	sprintf(buf, "AT+CHLD=2%d", id);
+	snprintf(buf, sizeof(buf), "AT+CHLD=2%d", id);
 	calypso_template(vc, buf, cb, data);
 }
 
@@ -190,7 +195,7 @@ static void calypso_deflect(struct ofono_voicecall *vc,
 {
 	char buf[128];
 
-	sprintf(buf, "AT+CTFR=%s,%d", ph->number, ph->type);
+	snprintf(buf, sizeof(buf), "AT+CTFR=%s,%d", ph->number, ph->type);
 	calypso_template(vc, buf, cb, data);
 }
 
@@ -210,10 +215,10 @@ static void calypso_send_dtmf(struct ofono_voicecall *vc, const char *dtmf,
 		return;
 	}
 
-	s = sprintf(buf, "AT+VTS=\"%c\"", dtmf[0]);
+	s = sprintf(buf, "AT+VTS=%c", dtmf[0]);
 
 	for (i = 1; i < len; i++)
-		s += sprintf(buf + s, ";+VTS=\"%c\"", dtmf[i]);
+		s += sprintf(buf + s, ";+VTS=%c", dtmf[i]);
 
 	calypso_template(vc, buf, cb, data);
 	g_free(buf);
@@ -235,8 +240,6 @@ static void cpi_notify(GAtResult *result, gpointer user_data)
 	int validity;
 	struct ofono_call call;
 
-	dump_response("cpi_notify", TRUE, result);
-
 	g_at_result_iter_init(&iter, result);
 
 	if (!g_at_result_iter_next(&iter, "%CPI:"))
@@ -373,25 +376,28 @@ static void calypso_voicecall_initialized(gboolean ok, GAtResult *result,
 	struct ofono_voicecall *vc = user_data;
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 
-	ofono_debug("voicecall_init: registering to notifications");
+	DBG("voicecall_init: registering to notifications");
 
 	g_at_chat_register(vd->chat, "%CPI:", cpi_notify, FALSE, vc, NULL);
 
 	ofono_voicecall_register(vc);
 }
 
-static int calypso_voicecall_probe(struct ofono_voicecall *vc, unsigned int vendor,
-					void *data)
+static int calypso_voicecall_probe(struct ofono_voicecall *vc,
+					unsigned int vendor, void *data)
 {
 	GAtChat *chat = data;
 	struct voicecall_data *vd;
 
-	vd = g_new0(struct voicecall_data, 1);
-	vd->chat = chat;
+	vd = g_try_new0(struct voicecall_data, 1);
+	if (!vd)
+		return -ENOMEM;
+
+	vd->chat = g_at_chat_clone(chat);
 
 	ofono_voicecall_set_data(vc, vd);
 
-	g_at_chat_send(chat, "AT%CPI=3", NULL,
+	g_at_chat_send(vd->chat, "AT%CPI=3", NULL,
 				calypso_voicecall_initialized, vc, NULL);
 
 	return 0;
@@ -401,6 +407,9 @@ static void calypso_voicecall_remove(struct ofono_voicecall *vc)
 {
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 
+	ofono_voicecall_set_data(vc, NULL);
+
+	g_at_chat_unref(vd->chat);
 	g_free(vd);
 }
 
@@ -410,7 +419,8 @@ static struct ofono_voicecall_driver driver = {
 	.remove			= calypso_voicecall_remove,
 	.dial			= calypso_dial,
 	.answer			= calypso_answer,
-	.hangup			= calypso_hangup,
+	.hangup_all		= calypso_ath,
+	.hangup_active		= calypso_chup,
 	.hold_all_active	= calypso_hold_all_active,
 	.release_all_held	= calypso_release_all_held,
 	.set_udub		= calypso_set_udub,
